/*
    Turbo BDM Light - USB communication
    Copyright (C) 2005  Daniel Malik

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include "common_usb.h"
#include "log.h"
#include "hwdesc.h"
#include "libusb.h"
#include "CmdTable.h"
#include "Commands.h"

// The device's data endpoints
#define EP_OUT (0x02)
#define EP_IN  (0x81)

static struct usb_device *bdm_devs[10]= {NULL};	/* array pointing to all Turbo BDM Light devices found terminated by NULL pointer */
static usb_dev_handle    *bdm_devh    = NULL;

/* Provides low level USB functions which talk to TBDML hardware */

/* Initialisation */
void bdm_usb_init(void) {
	usb_init();	 /* init LIBUSB */
}

/* Find all TBDML devices attached to the computer */
void bdm_usb_find_devices(void) {
	struct usb_bus     *libusb_bus;
	struct usb_device  *libusb_dev;
  	struct usb_version *version;
	unsigned int       i=0;
	bdm_devs[0]=NULL;				/* terminate the list */
	version = usb_get_version();
	if (version == NULL) {
		/* something quite wrong... */
		return;
	}
	print("===========================================\r\n");
   print("USBLIB DLL version: %i.%i.%i.%i\r\n",version->dll.major,version->dll.minor,version->dll.micro,version->dll.nano);
	if (version->driver.major == -1) {
		/* driver not running! */
		print("USB: Driver not running\r\n");
		return;
	}
	print("USBLIB Driver version: %i.%i.%i.%i\r\n",version->driver.major, version->driver.minor,version->driver.micro, version->driver.nano);
	usb_find_busses();		/* enumerate all busses */
	usb_find_devices();		/* enumerate all devices */
	for (libusb_bus = usb_get_busses(); libusb_bus; libusb_bus = libusb_bus->next) {		/* scan through all busses */
    	for (libusb_dev = libusb_bus->devices; libusb_dev; libusb_dev = libusb_dev->next) {	/* scan through all devices */
         if (((libusb_dev->descriptor.idVendor==TBDML_VID)&&(libusb_dev->descriptor.idProduct==TBDML_PID)) ||
             ((libusb_dev->descriptor.idVendor==OSBDM_VID)&&(libusb_dev->descriptor.idProduct==OSBDM_PID))) {
				/* found a device */
				bdm_devs[i++]=libusb_dev;	/* add pointer to the device */
				bdm_devs[i]=NULL;			   /* terminate the list again */
				if (i>=(sizeof(bdm_devs)/sizeof(struct usb_device*))) 
				   return;	/* too many devices! */
			}
		}
	}
}

/* Returns number of attached TBDML devices */
unsigned int bdm_usb_cnt(void) {
	unsigned int i=0;
	while ((i<(sizeof(bdm_devs)/sizeof(struct usb_device*)))&&
	       (bdm_devs[i]!=NULL)) i++;	/* count the entries */
	return(i);
}

/* Open connection to device enumerated by find_tbdml_devices */
/*  returns 0 on success and 1 on error */
unsigned char bdm_usb_open(unsigned int device_no) {
	bdm_devh = usb_open(bdm_devs[device_no]);
	if (bdm_devh==NULL) 
	   return (1);
   print("===========================================\r\n");
	print("USB Device open\r\n");
	if (usb_set_configuration(bdm_devh,1)) 
	   return(1);  /* TBDML has only one valid configuration */
	print("USB Configuration set\r\n");
	if (usb_claim_interface(bdm_devh,0)) 
	   return(1);		/* TBDML has only 1 interface */
	print("USB Interface claimed\r\n");
	return (0);
}

/* closes connection to the currently open device */
void bdm_usb_close(void) {
	if (bdm_devh!=NULL) {
		usb_release_interface(bdm_devh,0);		/* release the interface */
	   print("===========================================\r\n");
		print("USB Interface released\r\n");
		usb_close(bdm_devh);						/* close the device */
		print("USB Device closed\r\n");
		bdm_devh=NULL;							/* indicate that no device is open */
	}
}

/* Sends a message+data to the TBDML/OSBDM device over EP0
 * No response is expected
 * 
 * Entry:  data[0]    = N, the number of bytes to send (excluding this byte)
 *         data[1]    = OSBDM command byte
 *         data[2..N] = parameter(s) for OSBDM command
 *  
 * Returns: 0 => USB transfer OK
 *          1 => USB transfer Failed
 * 
 * Since the EP0 transfer is unidirectional in this case, data returned by the
 *  device must be read in a separate transfer - this includes any status. 
*/
unsigned char bdm_usb_send_ep0(unsigned char * data) {
	unsigned char count = *data;		/* data count is the first byte of the message */
 	int rc;
	if (bdm_devh==NULL) {
		print("USB EP0 send: device not open\r\n");
		return(1);
	}
   print("============================\r\n");
   print("USB EP0 send (%s, size=%d):\r\n", getCommandName(data[1]), count);
   if (data[1] == CMD_DEBUG)
      print("Debug cmd = %s\r\n", getDebugCommandName(data[2]));
   if (data[1] == CMD_OPTION)
      print("Option cmd = %s\r\n", getOptionCommandName(data[2]));
	printDump(data, count+1);
	rc=usb_control_msg(bdm_devh, 
	                   0x40,                            // requestType=Vendor
	                   *(data+1),                       // request
	                   (*(data+2))+256*(*(data+3)),     // value
	                   (*(data+4))+256*(*(data+5)),     // index
	                   data+6,                          // data bytes
	                   (count>5)?(count-5):0,           // size (# of data bytes)
	                   TIMEOUT);
	if (rc<0) {
      print("USB EP0 send: Send failed (USB error = %d)\r\n", rc);
      return(rc);
   }
	return(0);
}


/* Sends a message of up to 5 bytes to the TBDML/OSBDM device over EP0.
 * An immediate response is expected
 * 
 * Entry:  data[0]    = N, the number of bytes to receive from the device
 *         data[1]    = OSBDM command byte
 *         data[2..5] = parameter(s) for OSBDM command
 *
 * Exit:   data[0]    = cmd response from OSBDM
 *         data[1..]  = data response from the device
 *    
 * Returns: 0 => USB transfer OK, valid response from device
 *          1 => USB transfer Failed, or unexpected response from device
 */
unsigned char bdm_usb_recv_ep0(unsigned char * data) {
   unsigned char count = data[0]+1;   /* data count is the first byte of the message */
                                      // +1 for possible debug byte
   unsigned char cmd   = data[1];     /* OSBDM/TBDML Command byte */
 	int rc;
	if (bdm_devh==NULL) {
		print("USB EP0 receive request: device not open\r\n");
		return(1);
	}
   print("============================\r\n");
	print("USB EP0 receive request (%s, size=%d):\r\n", getCommandName(data[1]), count);
   if (data[1] == CMD_DEBUG)
      print("Debug cmd = %s\r\n", getDebugCommandName(data[2]));
   if (data[1] == CMD_OPTION)
      print("Option cmd = %s\r\n", getOptionCommandName(data[2]));
	printDump(data,6);
   rc=usb_control_msg(bdm_devh, 
                      0xC0,                           // bmRequestType
                      cmd,                            // bRequest
                      (*(data+2))+256*(*(data+3)),    // wValue
                      (*(data+4))+256*(*(data+5)),    // wIndex
                      data,                           // ptr to data buffer
                      count,                          // wLength = # of data bytes
                      TIMEOUT                         // timeout
                      );
   if (rc<0) {
      print("USB EP0 receive: Transfer failed (USB error = %d)\r\n", rc);
      return(1);
   }
   if ((data[0] != 0) && (data[0] != cmd)) { // Hack to accept old error codes
      print("USB EP0 receive: Cmd Failed (%s):\r\n", getErrorName(data[0]));
      print("USB EP0 receive (size = %d, recvd = %d):\r\n", count, rc);
      printDump(data,rc);
      return 1;
   }
   data[0] = 0; // for compatibility
   print("USB EP0 receive (size = %d, recvd = %d):\r\n", count, rc);
   printDump(data,rc);
   data[0] = cmd; // for compatibility
   return(0);
}

#if 0
/* sends a message to the TBDML device over EP2 */
/* returns 0 on success and 1 on error */
/* data the device wants to return need to be read out in separate recv transaction */
unsigned char bdm_usb_send_epOut(unsigned char *data) {
	unsigned char * count = data;		/* data count is the first byte of the message */
 	int i;
 	
	if (bdm_devh==NULL) {
		print("USB EP2 send: device not open\r\n");
		return(1);
	}
	print("USB EP2 send:\r\n");
	printDump(data,(*count)+1);
	i=usb_bulk_write(bdm_devh, EP_OUT, data, (*count)+1, TIMEOUT);
	if (i<0) return(1); else return(0);
}

/* receives data from EP1 */
/* returns 0 on success and 1 on error */
/* data count in the message is number of bytes EXPECTED/REQUIRED from the device */
unsigned char bdm_usb_recv_epIn(unsigned char * data) {
	unsigned char count = *data;		/* data count is the first byte of the message */
 	int i;
 	
	if (bdm_devh==NULL) {
		print("USB EP1 receive request: device not open\r\n");
		return(1);
	}
	print("USB EP1 receive request:\r\n");
	printDump(data,6);
	i=usb_bulk_read(bdm_devh, EP_IN, data, count, TIMEOUT);
	print("USB EP1 receive (%d byte(s)):\r\n",i);
	printDump(data,count);
	if (i<0) return(1); else return(0);
}

#define USE_EP0 (1)

/* Sends a message to the TBDML device
 * Returns 0 on success and 1 on error
 * No response is expected
 * 
 * data[0]       = size
 * data[1..size] = data
*/
unsigned char bdm_usb_send(unsigned char *data) {
#ifdef USE_EP0
   return bdm_usb_send_ep0(data);
#else
   print("TBDML SEND request (%d):\r\n", data[1]);
   printDump(data,6);
   return bdm_usb_send_epOut(data);
#endif
}

/* Sends a message to the TBDML device which instruct the device
 *  to execute a command and return data.
 * Returns 0 on success and 1 on error
 * Data count in the message is number of bytes EXPECTED/REQUIRED from the device
 * the device will get the cmd number and 4 following data bytes
 * 
 * On Tx
 * data[0]        = size of data to receive
 * data[1]        = command[0], command number
 * data[2]        = command[1], command parameter
 * data[3]        = command[2], command parameter
 * data[4]        = command[3], command parameter
 * data[5]        = command[4], command parameter
 * 
 * On Rx (return)
 * data[0]        = size of data received
 * data[1..size]  = data
 * 
 */
unsigned char bdm_usb_recv(unsigned char * data) {
#ifdef USE_EP0
   return bdm_usb_recv_ep0(data);
#else
   unsigned char size;
#if 0
   unsigned char rc;
   print("===================================\r\n"
         "tbdml_usb_recv() request:\r\n");
   printDump(data,6);
   size = data[0];
   // Tx on EP2
   data[0] = 5;
   rc = bdm_usb_send_epOut(data);
   if (rc != 0)
      return rc;
   // Rx on EP1
   data[0] = size;
   return bdm_usb_recv_epIn(data);
#endif   
   size = data[0];
   // Tx on EP2
   data[0] = 5;
   print("===================================\r\n");
   print("tbdml_usb_recv() request (Tx, cmd=%d):\r\n", data[1]);
   printDump(data,6);
   if (usb_bulk_write(bdm_devh, EP_OUT, data, data[0], TIMEOUT)<0) {
      print("error: bulk write failed\n");
      usb_clear_halt(bdm_devh, EP_OUT);
      return(1);
   }
   // Rx on EP1
   data[0] = size;
   print("tbdml_usb_recv() request (Rx, %d bytes):\r\n", data[0]);
   if (usb_bulk_read(bdm_devh, EP_IN, data, data[0], 5000) <= 0) {
      print("error: bulk read failed\n");
      usb_clear_halt(bdm_devh, EP_IN);
      return(1);
   }
   printDump(data,6);
   return 0;
#endif
}
#endif
