#ifndef _BDM_H_
#define _BDM_H_

//=================================================================================
#define BDM_SYNC_REQ      350U // us - length of the longest possible SYNC REQUEST pulse (128 BDM cycles @ 400kHz = 320us plus some extra time) 
#define SYNC_TIMEOUT      460U // us - longest time after which the target should have completed a SYNC pulse (16+128+margin cycles @ 400kHz = 375us) 
#define ACKN_TIMEOUT      375U // us - longest time after which the target should produce ACKN pulse (150 cycles @ 400kHz = 375us) 
#define SOFT_RESET       1280U // us - longest time needed for soft reset of the BDM interface (512 BDM cycles @ 400kHz = 1280us)
#define RESET_SETTLE     3000U // us - time to wait for signals to settle in us, this should certainly be longer than the soft reset time
#define BKGD_WAIT        1000U // us - time to hold BKGD pin low after reset pin rise for special modes (allowance made for slow Reset rise)
#define VDD_RISE_TIME    2000U // us - minimum time to allow for controlled target Vdd rise
#define RESET_LENGTH      100U // ms - time of RESET assertion
#define RESET_WAIT        300U // ms - how long to wait for the reset pin to come high
#define RESET_RECOVERY     10U // ms - how long to wait after reset before new commands are allowed

U8    bdm_init(U8 target);
U8    bdm_rx_tx_select(void);
void  bdm_tx_prepare(void);
void  bdm_tx_finish(void);
void  bdm_ackn(void);
void  bdm_wait64(void);
void  bdm_wait150(void);
void  bdm_ackn_init(void);
U8    bdm_hardware_reset(U8 mode); 
U8    bdm_target_reset( U8 mode);
U8    bdm_sync_meas(void);
U8    bdm_software_reset(U8 mode);
void  bdmRS08_Program_Flash(void);
void  bdmRS08_restoreTrim(void);
void  bdmRS08_mass_erase(void);
U8    bdm_connect(void);
U8    bdm_enableBDM(void);
U8    bdm_readBDMStatus(U8 *status);
U8    bdm_writeBDMControl(U8 bdm_sts);
void  bdm_checkTiming(void);
void  bdm_checkWaitTiming(void);

U8    bdmHC12_confirm_speed(U16 syncValue);
U8    bdm_make_active_if_stopped(void);
U8    bdm_cycle_Target_Vdd(void);
U16   bdm_Target_Vdd_measure(void);
void  bdm_suspend(void);
void  bdm_off( void );
void  bdm_halt(void);
void  bdm_go(void);
void  bdm_step(void);

typedef enum {
   HC12  = 0,
   HCS08 = 1,
   RS08  = 2,
   CFV1  = 3,
   T_OFF = 0xFF,
} target_type_e;

typedef enum {
   WAIT  = 0,   /* use WAIT instead */
   ACKN  = 1    /* ACKN feature available and enabled */
} ackn_e;

typedef enum {
   NO_RESET_ACTIVITY    =0,
   RESET_DETECTED       =1
} reset_e;

typedef enum {
   NO_INFO              =0,
   SYNC_SUPPORTED       =1,
   SPEED_GUESSED        =2,
   SPEED_USER_SUPPLIED  =3
} speed_e;

typedef struct {
   U8  initialised:1;
   U8  target_type:2;      /* target_type_e */
   U8  ackn:1;             /* ackn_e */
   U8  reset:1;            /* reset_e */
   U8  speed:2;            /* speed_e */
   U8  wait150_cnt;        /* time of 150 BDM cycles in bus cycles of the MCU divided by 3 */
   U8  wait64_cnt;  
   U16 sync_length;        /* length of the target SYNC pulse in 60MHz ticks */
} bdm_status_t;

typedef struct {
   U8  targetVdd:2;           // Target Vdd (off, 3.3V or 5V)
   U8  cycleVddOnReset:1;     // Cycle target Power  when resetting
   U8  cycleVddOnConnect:1;   // Cycle target Power if connection problems (when resetting?)
   U8  guessSpeed:1;          // Guess speed for target w/o ACKN
   U8  autoReconnect:1;       // Automatically re-connect to target (for speed change - experimental)
   U8  leaveTargetPowered:1;  // Leave target power on exit
   U8  useAltBDMClock:1;      // Use alternative BDM clock source in target (HCS08)
   U8  useResetSignal:1;      // Don't use RESET signal on BDM interface
} bdm_option_t;

typedef enum {
   FLASH_IDLE    = 0,   // No programming in progress, VPP is off
   FLASH_READY   = 2,   // Ready to program next block
   FLASH_BUSY    = 4,   // Busy programming a block
} flashState_t;

extern void bdm_txEmpty(U8 data);
extern U8   bdm_rxEmpty(void);

#pragma DATA_SEG __SHORT_SEG Z_PAGE

// Pointers to Rx & Tx routines
extern U8   (*bdm_rx_ptr)(void);
extern void (*bdm_tx_ptr)(U8);
#define BDM_TXRS_SET ((bdm_rx_ptr != bdm_rxEmpty) && (bdm_tx_ptr != bdm_txEmpty))

extern bdm_status_t bdm_status;
extern bdm_option_t bdm_option;
                        
#pragma DATA_SEG DEFAULT
extern volatile U8 flashState;

#define MAX_FLASH_PROGRAM (64)  // Maximum # of bytes to program into RS08 Flash in a single transaction
extern U8 flashProgrammingBuffer[5 + MAX_FLASH_PROGRAM]; // 5 byte header + data to program

#define bdm_rx()      (*bdm_rx_ptr)() 
#define bdm_tx(data)  (*bdm_tx_ptr)(data) 

/* hardware commands */
#define _BDM_BACKGROUND		      0x90
#define _BDM_ACK_ENABLE		      0xD5
#define _BDM_ACK_DISABLE	      0xD6
#define _BDM_READ_BYTE		      0xE0
#define _BDM_WRITE_BYTE		      0xC0
#define _BDM_WRITE_BLOCK	      0x88

/* HC/S12(x) hardware commands */
#define _BDM12_READ_BD_BYTE	   0xE4
#define _BDM12_READ_BD_WORD	   0xEC
#define _BDM12_READ_WORD	      0xE8
#define _BDM12_WRITE_BD_BYTE     0xC4
#define _BDM12_WRITE_BD_WORD     0xCC
#define _BDM12_WRITE_WORD	      0xC8

/* HCS08 'hardware'(non-intrusive) commands */
#define _BDM08_READ_STATUS	      0xE4
#define _BDM08_WRITE_CONTROL     0xC4
#define _BDM08_READ_BYTE_WS	   0xE1
#define _BDM08_READ_LAST	      0xE8
#define _BDM08_WRITE_BYTE_WS     0xC1
#define _BDM08_READ_BKPT	      0xE2
#define _BDM08_WRITE_BKPT        0xC2

/* firmware commands */
#define _BDM_GO				      0x08
#define _BDM_TRACE1              0x10
#define _BDM_TAGGO			      0x18

/* HCS08 'firmware' (active background mode) commands */
#define _BDM08_READ_A		      0x68
#define _BDM08_READ_CCR		      0x69
#define _BDM08_READ_PC		      0x6B
#define _BDM08_READ_HX		      0x6C
#define _BDM08_READ_SP		      0x6F
#define _BDM08_READ_NEXT	      0x70
#define _BDM08_READ_NEXT_WS 	   0x71
#define _BDM08_WRITE_A		      0x48
#define _BDM08_WRITE_CCR	      0x49
#define _BDM08_WRITE_PC		      0x4B
#define _BDM08_WRITE_HX		      0x4C
#define _BDM08_WRITE_SP	    	  	0x4F
#define _BDM08_WRITE_NEXT		 	0x50
#define _BDM08_NEXT_WS		      0x51
                           
#define _BDMRS08_WRITE_SPC		   0x4F

/* HC/S12(x) firmware commands */
#define _BDM12_READ_NEXT	      0x62
#define _BDM12_READ_PC		      0x63
#define _BDM12_READ_D		      0x64
#define _BDM12_READ_X		      0x65
#define _BDM12_READ_Y		      0x66
#define _BDM12_READ_SP		      0x67
#define _BDM12_WRITE_NEXT		   0x42
#define _BDM12_WRITE_PC		      0x43
#define _BDM12_WRITE_D		      0x44
#define _BDM12_WRITE_X		      0x45
#define _BDM12_WRITE_Y		      0x46
#define _BDM12_WRITE_SP		 	   0x47
#define _BDM12_GO_UNTIL			   0x0C
                               
// RS08 BDM Commands           
//=======================================
#define RS_BDC_RESET (0x18)

// Coldfire V1  Commands          
//=======================================
#define CF_ACK_DISABLE        (0x03) //
#define CF_ACK_ENABLE         (0x02) //
#define CF_BACKGROUND         (0x04) //
#define CF_DUMP_MEM           (0x32) //
#define CF_DUMP_MEM_WS        (0x33)
#define CF_FILL_MEM           (0x12) //
#define CF_FILL_MEM_WS        (0x13)
#define CF_GO                 (0x08) //
#define CF_NOP                (0x00) //
#define CF_READ_CREG          (0xE0) //
#define CF_READ_DREG          (0xA0) //
#define CF_READ_MEM           (0x30) // 
#define CF_READ_MEM_WS        (0x31) 
#define CF_READ_PSTB          (0x50) //
#define CF_READ_Rn            (0x60) //
#define CF_READ_XCSR_BYTE     (0x2D) //
#define CF_READ_CSR2_BYTE     (0x2E) //
#define CF_READ_CSR3_BYTE     (0x2F) //
#define CF_SYNC_PC            (0x01) //
#define CF_WRITE_CREG         (0xC0) //
#define CF_WRITE_DREG         (0x80) //
#define CF_WRITE_MEM          (0x10) //
#define CF_WRITE_MEM_WS       (0x11)
#define CF_WRITE_Rn           (0x40) //
#define CF_WRITE_XCSR_BYTE    (0x0D) //
#define CF_WRITE_CSR2_BYTE    (0x0E) //
#define CF_WRITE_CSR3_BYTE    (0x0F) //

#define CF_SZ_BYTE (0x0<<2)
#define CF_SZ_WORD (0x1<<2)
#define CF_SZ_LONG (0x2<<2)

/*
** The following macros perform individual BDM command
** Naming convention: bdm_cmd_x_y, 
**     where x in number of input parameters,
**           y is number of output paramreters
** suffxes indicate Word or Byte width
** HC/S12(X) wait times (in case ACKN is not used) are longer than those required for HCS08,
** so HCS08 can use the same macros 
** HCS08 should support ACKN anyway
*/

//============================================================
// The following commands DO NOT expect an ACK & do not delay
//  They leave the interface in the Tx condition (ready to drive )
//
// Write command byte, truncated sequence
#define BDM_CMD_0_0_T(cmd) \
   bdm_tx_prepare(),\
   bdm_tx(cmd)
   // bdm_tx_finish()

// Special for Software Reset HCS08, truncated sequence
#define BDM_CMD_1W1B_0_T(cmd, parameter1, parameter2) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)(((parameter1)>>8)&0xFF)),\
   bdm_tx((U8)((parameter1)&0xFF)),\
   bdm_tx((U8)((parameter2)&0xFF))
   // bdm_tx_finish()

// Special truncated sequence
#define BDM_CMD_1B_0_T(cmd,parameter)	\
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((parameter)&0xFF))
   
//============================================================
// The following commands DO NOT expect an ACK & do not delay
//
   /* write cmd without ACK ( HCS08/RS08/CFV1   ) */
#define BDM_CMD_0_0_NOACK(cmd) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx_finish()

   /* write cmd & read byte without ACK ( HCS08   ) */
#define BDM_CMD_0_1B_NOACK(cmd, result) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx_finish(),*((U8 *)((void *)(result)))=bdm_rx()

   /* write cmd  & byte without ACK (HCS08) */
#define BDM_CMD_1B_0_NOACK(cmd,parameter)	\
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((parameter)&0xFF)),\
   bdm_tx_finish()

/* write cmd & read word without ACK */
#define BDM_CMD_0_1W_NOACK(cmd,parameter) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx_finish(),\
   *((U8 *)((void *)(parameter)))=bdm_rx(), \
   *((U8 *)((void *)(parameter))+1)=bdm_rx()

/* write cmd & word  without ACK */
#define BDM_CMD_1W_0_NOACK(cmd,parameter) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((parameter)>>8)&0xFF),\
   bdm_tx((U8)((parameter)&0xFF)),\
   bdm_tx_finish()

//===============================================================
// The following DO expect an ACK or wait at end of the command 

/* no parameter */
#define BDM_CMD_0_0(cmd) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait64()
                                                         
/* write word */
#define BDM_CMD_1W_0(cmd,parameter) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((parameter)>>8)&0xFF),\
   bdm_tx((U8)((parameter)&0xFF)),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait64()

/* read word */
#define BDM_CMD_0_1W(cmd,parameter) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)? \
         bdm_ackn(): \
         bdm_wait64(), \
   *((U8 *)(parameter))=bdm_rx(), \
   *((U8 *)(parameter)+1)=bdm_rx()

/* write longword */
#define BDM_CMD_1L_0(cmd,parameter) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((parameter)>>24)&0xFF),\
   bdm_tx((U8)((parameter)>>16)&0xFF),\
   bdm_tx((U8)((parameter)>>8)&0xFF),\
   bdm_tx((U8)((parameter)&0xFF)),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait64()

/* write Little-endian longword*/
#define BDM_CMD_1Le_0(cmd,parameter) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((parameter)&0xFF)),\
   bdm_tx((U8)((parameter)>>8)&0xFF),\
   bdm_tx((U8)((parameter)>>16)&0xFF),\
   bdm_tx((U8)((parameter)>>24)&0xFF),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait64()

/* read longword */
#define BDM_CMD_0_1L(cmd,parameter) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)? \
         bdm_ackn(): \
         bdm_wait64(), \
   *((U8 *)(parameter)+0)=bdm_rx(), \
   *((U8 *)(parameter)+1)=bdm_rx(), \
   *((U8 *)(parameter)+2)=bdm_rx(), \
   *((U8 *)(parameter)+3)=bdm_rx()

/* read Little-endian longword */
#define BDM_CMD_0_1Le(cmd,parameter) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)? \
         bdm_ackn(): \
         bdm_wait64(), \
   *((U8 *)(parameter)+3)=bdm_rx(), \
   *((U8 *)(parameter)+2)=bdm_rx(), \
   *((U8 *)(parameter)+1)=bdm_rx(), \
   *((U8 *)(parameter)+0)=bdm_rx()

/* write word & read byte (read word but return byte - HC/S12(x)) */
#define BDM_CMD_1W_1WB(cmd, parameter, result) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)(((parameter)>>8)&0xFF)),\
   bdm_tx((U8)((parameter)&0xFF)),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait150();\
   if ((parameter)&0x0001) { \
      (void)bdm_rx();\
      *((U8 *)(result))=bdm_rx();\
   } else { \
      *((U8 *)(result))=bdm_rx();\
      (void)bdm_rx();\
   }

/* write 2 words */
#define BDM_CMD_2W_0(cmd, parameter1, parameter2) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((parameter1)>>8)&0xFF),\
   bdm_tx((U8)((parameter1)&0xFF)),\
   bdm_tx((U8)((parameter2)>>8)&0xFF),\
   bdm_tx((U8)((parameter2)&0xFF)),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait150()

/* write word and a byte (sends 2 words, the byte in both high and low byte of the 16-bit value) */
#define BDM_CMD_2WB_0(cmd, parameter1, parameter2) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((parameter1)>>8)&0xFF),\
   bdm_tx((U8)((parameter1)&0xFF)),\
   bdm_tx((U8)((parameter2)&0xFF)),\
   bdm_tx((U8)((parameter2)&0xFF)),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait150()

/* write word & read word */
#define BDM_CMD_1W_1W(cmd, parameter, result) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((parameter)>>8)&0xFF),\
   bdm_tx((U8)((parameter)&0xFF)),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait150(),\
   *((U8 *) ((void *)(result)))=bdm_rx(),\
   *((U8 *) ((void *)(result))+1)=bdm_rx()

/* read word HCS08 */
#define BDM_CMD_0_1B(cmd,result)	\
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait64(),\
   *((U8 *)((void *)(result)))=bdm_rx()
                                                
/* write byte HCS08 */
#define BDM_CMD_1B_0(cmd,parameter)	\
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((parameter)&0xFF)),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait64()                                                

/* write word & read byte (read word but return byte - HCS08 */
#define BDM_CMD_1W_1B(cmd, parameter, result)  \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((parameter)>>8)&0xFF),\
   bdm_tx((U8)((parameter)&0xFF)),\
   bdm_tx_finish(), \
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait64(), \
   *((U8 * far)((void * far)(result)))=(U8)bdm_rx()									                                                           									
                                                         
/* write 1 words then delay then 1byte HCS08 */
#define BDM_CMD_1W1B_0(cmd, parameter1, parameter2) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((parameter1)>>8)&0xFF),\
   bdm_tx((U8)((parameter1)&0xFF)),\
   bdm_tx((U8)((parameter2))&0xFF),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait150()                              									

/* write 1 24-bit value then 1 byte  */
#define BDM_CMD_1A1B_0(cmd, addr, value) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((addr)>>16)&0xFF),\
   bdm_tx((U8)((addr)>>8)&0xFF),\
   bdm_tx((U8)((addr)&0xFF)),\
   bdm_tx((U8)((value))&0xFF),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait150()                              									

/* write 1 24-bit value then 1 word  */
#define BDM_CMD_1A1W_0(cmd, addr, value) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((addr)>>16)&0xFF),\
   bdm_tx((U8)((addr)>>8)&0xFF),\
   bdm_tx((U8)((addr)&0xFF)),\
   bdm_tx((U8)((value)>>8)&0xFF),\
   bdm_tx((U8)((value))&0xFF),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait150()                              									

/* write 1 24-bit value then 1 longword  */
#define BDM_CMD_1A1L_0(cmd, addr, value) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((addr)>>16)&0xFF),\
   bdm_tx((U8)((addr)>>8)&0xFF),\
   bdm_tx((U8)((addr)&0xFF)),\
   bdm_tx((U8)((value)>>24)&0xFF),\
   bdm_tx((U8)((value)>>16)&0xFF),\
   bdm_tx((U8)((value)>>8)&0xFF),\
   bdm_tx((U8)((value))&0xFF),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait150()                              									

/* write 24-bit address & read byte */
#define BDM_CMD_1A_1B(cmd, addr, result) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((addr)>>16)&0xFF),\
   bdm_tx((U8)((addr)>>8)&0xFF),\
   bdm_tx((U8)((addr)&0xFF)),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait150(),\
         *((U8 *) ((void *)(result)))=bdm_rx()

/* write 24-bit address & read word */
#define BDM_CMD_1A_1W(cmd, addr, result) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((addr)>>16)&0xFF),\
   bdm_tx((U8)((addr)>>8)&0xFF),\
   bdm_tx((U8)((addr)&0xFF)),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait150(),\
         *((U8 *) ((void *)(result)))=bdm_rx(),\
         *((U8 *) ((void *)(result))+1)=bdm_rx()

/* write 24-bit address & read longword */
#define BDM_CMD_1A_1L(cmd, addr, result) \
   bdm_tx_prepare(),\
   bdm_tx(cmd),\
   bdm_tx((U8)((addr)>>16)&0xFF),\
   bdm_tx((U8)((addr)>>8)&0xFF),\
   bdm_tx((U8)((addr)&0xFF)),\
   bdm_tx_finish(),\
   (bdm_status.ackn==ACKN)?bdm_ackn():bdm_wait150(),\
         *((U8 *) ((void *)(result)))=bdm_rx(),\
         *((U8 *) ((void *)(result))+1)=bdm_rx(),\
         *((U8 *) ((void *)(result))+2)=bdm_rx(),\
         *((U8 *) ((void *)(result))+3)=bdm_rx()

/* Definitions of the actual commands - these should be used in the C-code
 * All BDM commands need to be called with interrupts disabled (either by
 *  hand or, for example, from within an ISR)
 * value is 8 or 16 bit value, addr is 16 bit address, 
 * value_p is pointer to 8 or 16 bit variable */

/* FIRMWARE COMMANDS */
#define BDM_CMD_GO()				            BDM_CMD_0_0(_BDM_GO)
#define BDM12_CMD_GO_UNTIL()		         BDM_CMD_0_0(_BDM12_GO_UNTIL)
#define BDM_CMD_TRACE1()			         BDM_CMD_0_0(_BDM_TRACE1)
#define BDM_CMD_TAGGO()				         BDM_CMD_0_0(_BDM_TAGGO)
/* write memory using X as a pointer with automatic pre-increment */ 
#define BDM12_CMD_WRITE_NEXT(value)		   BDM_CMD_1W_0(_BDM12_WRITE_NEXT,value)
/* write register commands */
#define BDM12_CMD_WRITE_PC(value)	      BDM_CMD_1W_0(_BDM12_WRITE_PC,value)
#define BDM12_CMD_WRITE_D(value)		      BDM_CMD_1W_0(_BDM12_WRITE_D,value)
#define BDM12_CMD_WRITE_X(value)		      BDM_CMD_1W_0(_BDM12_WRITE_X,value)
#define BDM12_CMD_WRITE_Y(value)		      BDM_CMD_1W_0(_BDM12_WRITE_Y,value)
#define BDM12_CMD_WRITE_SP(value)		   BDM_CMD_1W_0(_BDM12_WRITE_SP,value)
/* read memory using X as a pointer with automatic pre-increment */
#define BDM12_CMD_READ_NEXT(value_p)	   BDM_CMD_0_1W(_BDM12_READ_NEXT,value_p)
/* read register commands */
#define BDM12_CMD_READ_PC(value_p)		   BDM_CMD_0_1W(_BDM12_READ_PC,value_p)
#define BDM12_CMD_READ_D(value_p)		   BDM_CMD_0_1W(_BDM12_READ_D,value_p)
#define BDM12_CMD_READ_X(value_p)		   BDM_CMD_0_1W(_BDM12_READ_X,value_p)
#define BDM12_CMD_READ_Y(value_p)		   BDM_CMD_0_1W(_BDM12_READ_Y,value_p)
#define BDM12_CMD_READ_SP(value_p)		   BDM_CMD_0_1W(_BDM12_READ_SP,value_p)
/* HARDWARE COMMANDS */
#define BDM_CMD_ACK_ENABLE()			      BDM_CMD_0_0(_BDM_ACK_ENABLE)
#define BDM_CMD_ACK_DISABLE() 			   BDM_CMD_0_0(_BDM_ACK_DISABLE)  // Does timeout on ACK
#define BDM_CMD_BACKGROUND()			      BDM_CMD_0_0(_BDM_BACKGROUND)
/* read and write commands */
#define BDM12_CMD_WRITEW(addr,value)	   BDM_CMD_2W_0(_BDM12_WRITE_WORD,addr,value)
#define BDM12_CMD_WRITEB(addr,value)	   BDM_CMD_2WB_0(_BDM_WRITE_BYTE,addr,value)
#define BDM12_CMD_READW(addr,value_p)	   BDM_CMD_1W_1W(_BDM12_READ_WORD,addr,value_p)
#define BDM12_CMD_READB(addr,value_p)	   BDM_CMD_1W_1WB(_BDM_READ_BYTE,addr,value_p)
/* read and writes from/to the BDM memory space */
#define BDM12_CMD_BDWRITEW(addr,value)	   BDM_CMD_2W_0(_BDM12_WRITE_BD_WORD,addr,value)
#define BDM12_CMD_BDWRITEB(addr,value)    BDM_CMD_2WB_0(_BDM12_WRITE_BD_BYTE,addr,value)
#define BDM12_CMD_BDREADW(addr,value_p)   BDM_CMD_1W_1W(_BDM12_READ_BD_WORD,addr,value_p)
#define BDM12_CMD_BDREADB(addr,value_p)   BDM_CMD_1W_1WB(_BDM12_READ_BD_BYTE,addr,value_p)

/* read register commands */
#define BDM08_CMD_READSTATUS(value_p)     BDM_CMD_0_1B_NOACK(_BDM08_READ_STATUS,value_p) // No ACK fix - pgo
#define BDM08_CMD_WRITECONTROL(value)     BDM_CMD_1B_0_NOACK(_BDM08_WRITE_CONTROL,value) // No ACK fix - pgo
#define BDM08_CMD_READB(addr,value_p) 	   BDM_CMD_1W_1B(_BDM_READ_BYTE,addr,value_p)
#define BDM08_CMD_WRITEB(addr,value)	   BDM_CMD_1W1B_0(_BDM_WRITE_BYTE,addr,value) 
#define BDM08_CMD_RESET()                 BDM_CMD_1W1B_0_T(_BDM_WRITE_BYTE,HCS08_SBDFR, HCS_SBDFR_BDFR)

#define BDM08_CMD_WRITEBLOCK(addr,value)	BDM_CMD_1W1B_0(_BDM_WRITE_BLOCK,addr,value) 
#define BDMRS08_CMD_RESET() 			      BDM_CMD_0_0_T(RS_BDC_RESET)

/* read register commands */
#define BDM08_CMD_READ_PC(value_p)		   BDM_CMD_0_1W(_BDM08_READ_PC,value_p)
#define BDM08_CMD_READ_SP(value_p)		   BDM_CMD_0_1W(_BDM08_READ_SP,value_p)
#define BDM08_CMD_READ_HX(value_p)		   BDM_CMD_0_1W(_BDM08_READ_HX,value_p)
#define BDM08_CMD_READ_A(value_p)		   BDM_CMD_0_1B(_BDM08_READ_A,value_p)
#define BDM08_CMD_READ_CCR(value_p)		   BDM_CMD_0_1B(_BDM08_READ_CCR,value_p)
#define BDM08_CMD_READ_BKPT(value_p) 		BDM_CMD_0_1W_NOACK(_BDM08_READ_BKPT,value_p) // No ACK fix - pgo
 
/* write register commands */
#define BDM08_CMD_WRITE_PC(value)		   BDM_CMD_1W_0(_BDM08_WRITE_PC,value)
#define BDM08_CMD_WRITE_SP(value)		   BDM_CMD_1W_0(_BDM08_WRITE_SP,value)
#define BDM08_CMD_WRITE_HX(value)		   BDM_CMD_1W_0(_BDM08_WRITE_HX,value)
#define BDM08_CMD_WRITE_A(value)		      BDM_CMD_1B_0(_BDM08_WRITE_A,value)
#define BDM08_CMD_WRITE_CCR(value)		   BDM_CMD_1B_0(_BDM08_WRITE_CCR,value)
#define BDM08_CMD_WRITE_BKPT(value)		   BDM_CMD_1W_0_NOACK(_BDM08_WRITE_BKPT,value) // No ACK fix - pgo

#define BDMRS08_CMD_WRITE_SPC(value)		BDM_CMD_1W_0(_BDMRS08_WRITE_SPC,value)

/* write memory using X as a pointer with automatic pre-increment */ 
#define BDM08_CMD_WRITE_NEXT(value)		   BDM_CMD_1B_0(_BDM08_WRITE_NEXT,value)
/* read memory using X as a pointer with automatic pre-increment */
#define BDM08_CMD_READ_NEXT(value_p)	   BDM_CMD_0_1B(_BDM08_READ_NEXT,value_p)

// Coldfire V1 Commands
#define BDMCF_CMD_ACK_ENABLE()		       BDM_CMD_0_0(CF_ACK_ENABLE)
#define BDMCF_CMD_ACK_DISABLE() 			    BDM_CMD_0_0(CF_ACK_DISABLE)  // Does timeout on No ACK
#define BDMCF_CMD_BACKGROUND()		       BDM_CMD_0_0(CF_BACKGROUND)
#define BDMCF_CMD_GO()				          BDM_CMD_0_0(CF_GO)
#define BDMCF_CMD_NOP()				          BDM_CMD_0_0(CF_NOP)
#define BDMCF_CMD_SYNC_PC()                BDM_CMD_0_0(CF_SYNC_PC)

#define BDMRS08_CMD_RESET() 			       BDM_CMD_0_0_T(RS_BDC_RESET)

#define BDMCF_CMD_READ_REGe(regNo,value_p)  BDM_CMD_0_1Le(CF_READ_Rn|regNo,value_p)
#define BDMCF_CMD_READ_CREGe(regNo,value_p) BDM_CMD_0_1Le(CF_READ_CREG|regNo,value_p)
#define BDMCF_CMD_READ_DREGe(regNo,value_p) BDM_CMD_0_1Le(CF_READ_DREG|regNo,value_p)
#define BDMCF_CMD_READ_PSTBe(regNo,value_p) BDM_CMD_0_1Le(CF_READ_PSTB|regNo,value_p)
#define BDMCF_CMD_READ_DREG(regNo,value_p)  BDM_CMD_0_1L(CF_READ_DREG|regNo,value_p)

#define BDMCF_CMD_READ_XCSR(value_p)       BDM_CMD_0_1B_NOACK(CF_READ_XCSR_BYTE,value_p) // No ACK
#define BDMCF_CMD_READ_CSR2(value_p)       BDM_CMD_0_1B_NOACK(CF_READ_CSR2_BYTE,value_p) // No ACK
#define BDMCF_CMD_READ_CSR3(value_p)       BDM_CMD_0_1B_NOACK(CF_READ_CSR3_BYTE,value_p) // No ACK

#define BDMCF_CMD_WRITE_REGe(regNo,value)  BDM_CMD_1Le_0(CF_WRITE_Rn|regNo,value)
#define BDMCF_CMD_WRITE_CREGe(regNo,value) BDM_CMD_1Le_0(CF_WRITE_CREG|regNo,value)
#define BDMCF_CMD_WRITE_DREGe(regNo,value) BDM_CMD_1Le_0(CF_WRITE_DREG|regNo,value)
#define BDMCF_CMD_WRITE_DREG(regNo,value)  BDM_CMD_1L_0(CF_WRITE_DREG|regNo,value)

#define BDMCF_CMD_WRITE_XCSR(value)        BDM_CMD_1B_0_NOACK(CF_WRITE_XCSR_BYTE,value) // No ACK
#define BDMCF_CMD_WRITE_CSR2(value)        BDM_CMD_1B_0_NOACK(CF_WRITE_CSR2_BYTE,value) // No ACK
#define BDMCF_CMD_WRITE_CSR3(value)        BDM_CMD_1B_0_NOACK(CF_WRITE_CSR3_BYTE,value) // No ACK

#define BDMCF_CMD_READ_MEM_B(addr,value_p) BDM_CMD_1A_1B(CF_READ_MEM|CF_SZ_BYTE,addr,value_p)
#define BDMCF_CMD_READ_MEM_W(addr,value_p) BDM_CMD_1A_1W(CF_READ_MEM|CF_SZ_WORD,addr,value_p)
#define BDMCF_CMD_READ_MEM_L(addr,value_p) BDM_CMD_1A_1L(CF_READ_MEM|CF_SZ_LONG,addr,value_p)

#define BDMCF_CMD_DUMP_MEM_B(value_p) BDM_CMD_0_1B(CF_DUMP_MEM|CF_SZ_BYTE,value_p)
#define BDMCF_CMD_DUMP_MEM_W(value_p) BDM_CMD_0_1W(CF_DUMP_MEM|CF_SZ_WORD,value_p)
#define BDMCF_CMD_DUMP_MEM_L(value_p) BDM_CMD_0_1L(CF_DUMP_MEM|CF_SZ_LONG,value_p)

#define BDMCF_CMD_WRITE_MEM_B(addr,value) BDM_CMD_1A1B_0(CF_WRITE_MEM|CF_SZ_BYTE,addr,value)
#define BDMCF_CMD_WRITE_MEM_W(addr,value) BDM_CMD_1A1W_0(CF_WRITE_MEM|CF_SZ_WORD,addr,value)
#define BDMCF_CMD_WRITE_MEM_L(addr,value) BDM_CMD_1A1L_0(CF_WRITE_MEM|CF_SZ_LONG,addr,value)

#define BDMCF_CMD_FILL_MEM_B(value) BDM_CMD_1B_0(CF_FILL_MEM|CF_SZ_BYTE,value)
#define BDMCF_CMD_FILL_MEM_W(value) BDM_CMD_1W_0(CF_FILL_MEM|CF_SZ_WORD,value)
#define BDMCF_CMD_FILL_MEM_L(value) BDM_CMD_1L_0(CF_FILL_MEM|CF_SZ_LONG,value)

#endif
