/*******************************************************************
 *
 * ICP Boot Code
 *
 *  The following code is placed in a region of flash that is not
 *  erased by default.  It detects the need for ICP and also 
 *  redirects the vector table to the user vector table.
 *
 *  Loosely Based on Freescale Application Note: AN2399SW
 *******************************************************************/
 
 /**********************************************************************
 *  WARNING
 *
 *  It is very important that this code does not use any library
 *  routines as they will end up in the USER ROM area which may be replaced
 *  This code must be stand-alone.
 *
 **********************************************************************/
#include "derivative.h" // Include peripheral declarations
#include "ICP.h"

#define MAJOR_VER (1)
#define MINOR_VER (0)

const int ICPVersion @ ICP_VERSION_LOCATION = MAJOR_VER<<8|MINOR_VER;

/*
**  User flash area dimensions
*/
#define USER_FLASH_START (0xBA00)
#define USER_FLASH_END   (0xF7FF)
#define FLBPR_VALUE      ((USER_FLASH_END+1)>>8) // ICP Area is protected

/*
** All this code goes in ICP boot area
*/
#pragma CODE_SEG ICP_ROM

/******************************************************************
 *  Data structure for Flash programming routines in ROM
 ******************************************************************/
typedef struct {
   unsigned char       ctrlbyt; // Control byte (mass/block erase)
   unsigned char       cpuspd;  // CPU speed 4x bus speed in MHz
   unsigned char * far laddr;   // Last Address to program
} ProgrammingInfo;

// Fixed memory locations used by Flash programming routines in ROM 
static volatile ProgrammingInfo programmingInfo @0x0088;
char flashBuffer[3] @0x100;

// Values for ctrlbyt
#define BLOCK_ERASE (0)
#define MASS_ERASE  (1<<6)

/*
 * Entry points to flash programming routines in ROM Monitor
 */
#define MON_VERIFY   (0xFC03) // FLASH verify routine
#define MON_ERASE    (0xFC06) // FLASH mass or block erase routine
#define MON_PROGRAM  (0xFC09) // FLASH program routine
#define MON_USB_ICP  (0xFA19) // Enter USB ICP

/*******************************************************
 * Forces ICP mode on next reset by re-programming
 * the user reset vector in the user flash area.
 *
 * This routine is intended to be called from the
 * user flash memory and protects the RAM it modifies.
 ******************************************************/
static void _forceICPOnReset(void) {
   ProgrammingInfo saveInfo;
   char saveBuffer[1];

   // Save existing contents of RAM   
// saveInfo.ctrlbyt = programmingInfo.ctrlbyt; 
   saveInfo.cpuspd  = programmingInfo.cpuspd;
   saveInfo.laddr   = programmingInfo.laddr;
   saveBuffer[0]    = flashBuffer[0];

   // Set up programming information
// programmingInfo.ctrlbyt = BLOCK_ERASE;
   programmingInfo.cpuspd  = 24;  // Bus Speed of 6 MHz
   programmingInfo.laddr   = ((char* far)userVectorTable+sizeof(userVectorTable)-2);
   flashBuffer[0] = 0;

   FLBPR=FLBPR_VALUE; // Protect vectors and boot loader & unprotect rest
   
   // Programs the first byte of the address in the reset entry of the user vector table to 0
   asm {
      ldhx    #@userVectorTable+49   // load the address to be programmed into H:X
      jsr     MON_PROGRAM            // call the flash programming routine in ROM
   }
   
   // Restore contents of RAM
// programmingInfo.ctrlbyt = saveRAM.ctrlbyt; 
   programmingInfo.cpuspd  = saveInfo.cpuspd;
   programmingInfo.laddr   = saveInfo.laddr;
   flashBuffer[0]          = saveBuffer[0];
}

/*************************************************************
 * ICP mode is indicted by a PTA.0 being low
 * This is failsafe ICP option during development.  
 * PTA.0 is not accessible in final version (shrink wrapped).
 *************************************************************/
static char detectICP(void) {
   enum {
      TSTPIN = 0x01,  // Test output
   };
   char doICP;
   
   POCR_PAP  = 1;               // Enable pull-up
   DDRA     &= ~TSTPIN;         // Make input
   asm {
      nop
      nop
   }
   doICP     = (~PTA & TSTPIN); // Check pin
   
   POCR_PAP  = 0;               // Restore reset state
   
   return doICP;
}

/*****************************************************************************************
 * Check if user region of flash is invalid
 *
 * First checks for invalid reset vector then does a simple checksum over the user flash.
 * See PRM file for example of automatic calculation of checksum
 ****************************************************************************************/
#pragma MESSAGE DISABLE C1404 // Disable warnings about return expected
static char flashInvalid(){
   asm{
      // Check for valid user reset vector
      lda      #1                       // Set up for failed test
      ldx      *(@userVectorTable+49)   // load the user code reset address
      pshx
      pulh
      ldx      *(@userVectorTable+50)
      cphx     #USER_FLASH_START        // exit if not pointing within flash
      blo      ex
      cphx     #USER_FLASH_END
      bhi      ex

      ldhx     #USER_FLASH_START
      clra
      
      // Sum the flash memory
   loop:
      sta      _COPCTL // Feed the watchdog
      add      ,x
      aix      #1
      cphx     #USER_FLASH_END
      blo      loop
      
      sub      ,x   // Subtract the checksum
                    // 0 => OK
   ex:
   }
}
#pragma MESSAGE DEFAULT C1404 // Restore warnings about return expected

/*************************************************************
 * Calls user ICP detection routine via fixed location vector
 *************************************************************/
#pragma NO_RETURN
#pragma MESSAGE DISABLE C1404 // Disable warnings about return expected
static char _userDetectICP(void){
   asm{
      sta _COPCTL         // Feed the watchdog
      jmp userVectorTable // First entry is userDetectICP()
   }
}
#pragma MESSAGE DEFAULT C1404 // Restore warnings about return expected

/*****************************************************************
 * Checks if ICP is required
 *   If so, ICP code in ROM is called
 *   otherwise user reset code is called via user vector table
 ****************************************************************/
#pragma TRAP_PROC
#pragma NO_RETURN
static void ICPDetect(void) {

   if (detectICP() || flashInvalid() || _userDetectICP()) {
      CONFIG = CONFIG_URSTD_MASK | CONFIG_COPD_MASK;
      DDRD |= 0x01;        // Turn on LED
      PTD  &= ~0x01;
      asm{
         jmp MON_USB_ICP   // Enter ICP ROM
      };                             
   }

   FLBPR=FLBPR_VALUE; // Protect vectors and boot loader

   // Jmp to user code via user reset vector
   asm{
      sta   _COPCTL             // Feed the watchdog
      jmp   userVectorTable:48  // jmp via reset vector
   }
}

/***********************************************************************
 * Hardware vector table - mostly redirects to user vector (jump) table 
 ***********************************************************************/

typedef void (* const vector)(void);
#define VECTOR(x) (vector)(x)

#pragma CONST_SEG VECTOR_TABLE
#pragma MESSAGE DISABLE C1805 // Disable warnings about non-standard conversion
const vector vectorTable[17] = {
   // Fixed vector used to provide access to routine in user flash area 
   VECTOR(_forceICPOnReset),       // Force ICP on next reset
   
   // Vectors apart from reset pass to user code via user vector table
   VECTOR(userVectorTable+1),      // Keyboard
   VECTOR(userVectorTable+2),      // SCI Transmit
   VECTOR(userVectorTable+3),      // SCI Receive
   VECTOR(userVectorTable+4),      // SCI Error
   VECTOR(userVectorTable+5),      // TIM2 Overflow
   VECTOR(userVectorTable+6),      // TIM2 Ch0 & Ch1
   VECTOR(userVectorTable+7),      // TIM2 Ch1
   VECTOR(userVectorTable+8),      // TIM2 Ch0
   VECTOR(userVectorTable+9),      // TIM1 Overflow
   VECTOR(userVectorTable+10),     // TIM1 Ch0 & Ch1
   VECTOR(userVectorTable+11),     // TIM1 Ch1
   VECTOR(userVectorTable+12),     // TIM1 Ch0
   VECTOR(userVectorTable+13),     // IRQ
   VECTOR(userVectorTable+14),     // USB
   VECTOR(userVectorTable+15),     // SWI
   
   // Reset is handler by ICP boot code
   ICPDetect                       // Reset
};
#pragma CONST_SEG DEFAULT
#pragma MESSAGE DEFAULT C1805 // Restore warnings about non-standard conversion
