/*
    Open Source BDM/Turbo BMD Light - main program loop
    Original TBDML Copyright (C) 2005  Daniel Malik
     **  Prominent Notice-This software was modified from TBDML software - 12/05  **

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "hidef.h"
#include "derivative.h"
#include "Common.h"
#include "Configure.h"
#include "commands.h"
#include "usb.h"
#include "bdm.h"
#include "led.h"
#include "main.h"
#include "ICP.h"

/*
   The basic program flow:
      1. all BDM activity is driven by commands received from the USB
      2. all BDM commands are executed from within the ISRs servicing the USB IRQ
         (i.e. with interrupts disabled)
      3. suspend timekeeping is performed in the main loop
      4. the only asynchronous activity is detection of external resets on RESET_IN
         pin through KBD interrupts
*/

/* timer counts up and the part is suspended when it reaches treshold value */
/* the USB interrupt resets the timer */

#pragma DATA_SEG DEFAULT
volatile signed char suspend_timer;

#if (DEBUG&STACK_DEBUG)

//===========================================================
//  Clear stack to allows probing to determine the used stack
//
void clearStack(void) {
extern char far __SEG_START_SSTACK[];  // bottom of stack space
//extern char far __SEG_END_SSTACK[];    // top of stack space

   asm {
      tsx                        // Current TOS
      aix   #-2                  // Allows space for value to be pushed
      pshx                       // Save on stack for comparison
      pshh
      ldhx  @__SEG_START_SSTACK  // Clear from bottom of stack
   loop:    
      clr   0,x                  // Clear next byte
      aix   #1 
      cpx   2,sp                 // Reached TOS?
      bne   loop                 // No - loop
      pshh
      pula
      cmp   1,sp
      bne   loop
      
      ais   #2                   // Clean up stack
   }
}
#else
#define clearStack() ;
#endif

/* initialise the CPU */
void init(void) {
   /* disable part reset on USB reset, enable STOP instruction & disable COP */
   CONFIG = CONFIG_URSTD_MASK | CONFIG_STOP_MASK | CONFIG_COPD_MASK;
   ISCR   = ISCR_ACK_MASK | ISCR_IMASK_MASK;   /* acknowledge IRQ interrupt and disable it */

   clearStack();   
   (void)bdm_init(HCS08);
   usb_init();
   EnableInterrupts;
}

/* wait 100us */
void wait100us(void) {
#define CYCLES_IN_100US (BUS_FREQ/10000UL)

   asm {
            LDA   #(CYCLES_IN_100US/3)-4-2-4  /* minus cycles needed for BSR, LDA and RTS */
      Loop: DBNZA Loop  /* 3 cycles per iteration */ 
   }
}

/*************************************************
 * ICP mode is indicated by a loopback
 * from BDM to RST pins of the BDM cable.
 * This routine is called DIRECTLY from the
 * ICP boot code - minimal C setup (stack only)!
 *************************************************/
char userDetectICP(void) {
char loopBackDetected;

   DATA_PORT_DDR = BDM_OUT_MASK; // Configure port directions
   DIR_PORT_DDR  = BDM_DIR_MASK|RESET_DIR_MASK;

   DIR_PORT      = BDM_DIR_WR|RESET_DIR_RD; // BDM driven, RESET input

   // Drive test pin high
   DATA_PORT     = BDM_OUT_MASK; // Drive BDM high
   asm{
      nop
      nop
   }
   // Check for loopback
   loopBackDetected = ((DATA_PORT & RESET_IN_MASK) != 0);

   // Drive test pin low
   DATA_PORT     = 0; // Drive BDM low
   asm{
      nop
      nop
   }
   // Check for loopback
   loopBackDetected = loopBackDetected && ((DATA_PORT & RESET_IN_MASK) == 0);

   // Make ports safe
   DATA_PORT_DDR = 0; // Configure port directions
   DIR_PORT_DDR  = 0;

   return loopBackDetected; // Loopback detected
}

/* main function */
void main(void) {

   init();
   for(;;){
      // I have no idea what these states mean - very poor code style!!
      if (flashState == FLASH_BUSY) {
         flashState = FLASH_READY;
      }
      wait100us();
      suspend_timer++;
      if (suspend_timer>=SUSPEND_TIME) {
         /* host is not sending keepalive signals, time to suspend all operation */
         /* BDM is in idle mode when not communicating, so nothing to do there */
         U16 i;
         /* Acknowledge any pending interrupt and disable KBD interrupts; */
         /* This will prevent RESET activity waking us up out of stop */
         KBSCR = KBSCR_IMASKK_MASK | KBSCR_ACKK_MASK;

         led_state=LED_OFF;  /* switch the LED off */
         GREEN_LED_OFF();       /* do it now, the interrupt which would do it normally is not going to come */

         UIR0_SUSPND=1;      /* suspend USB */
         while (suspend_timer) asm(STOP);  /* go to sleep, wait for USB resume or reset */

         for (i=0;i<RESUME_RECOVERY;i++) wait100us();  /* wait for host to recover */

         led_state=LED_ON;   /* Switch the LED back on */
         (void)bdm_init(HCS08);    /* Reinitialise the BDM after wake-up as the device might */
                             /* have been disconected in the meantime; assume nothing */
      }
   }
}
